@@ -2,9 +2,7 @@
  *
  * rasqal_general.c - Rasqal library startup, shutdown and factories
  *
- * $Id: rasqal_general.c 11816 2007-01-24 07:47:50Z dajobe $
- *
- * Copyright (C) 2004-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2009, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -44,21 +42,14 @@
 
 
 /* prototypes for helper functions */
-static void rasqal_delete_query_engine_factories(void);
+static void rasqal_delete_query_language_factories(rasqal_world*);
 
 
 /* statics */
-static int rasqal_initialised=0;
 
-static int rasqal_initialising=0;
-static int rasqal_finishing=0;
+const char * const rasqal_short_copyright_string = "Copyright 2003-2009 David Beckett.  Copyright 2003-2005 University of Bristol";
 
-/* list of query factories */
-static rasqal_query_engine_factory* query_engines=NULL;
-
-const char * const rasqal_short_copyright_string = "Copyright 2003-2007 David Beckett.  Copyright 2003-2005 University of Bristol";
-
-const char * const rasqal_copyright_string = "Copyright (C) 2003-2007 David Beckett - http://purl.org/net/dajobe/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
+const char * const rasqal_copyright_string = "Copyright (C) 2003-2009 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
 
 const char * const rasqal_license_string = "LGPL 2.1 or newer, GPL 2 or newer, Apache 2.0 or newer.\nSee http://librdf.org/rasqal/LICENSE.html for full terms.";
 
@@ -71,7 +62,7 @@ const char * const rasqal_home_url_string = "http://librdf.org/rasqal/";
  *
  * See also #rasqal_version_decimal.
  */
-const char * const rasqal_version_string = VERSION;
+const char * const rasqal_version_string = RASQAL_VERSION_STRING;
 
 /**
  * rasqal_version_major:
@@ -101,123 +92,251 @@ const unsigned int rasqal_version_release = RASQAL_VERSION_RELEASE;
  *
  * See also #rasqal_version_string.
  */
-const unsigned int rasqal_version_decimal = RASQAL_VERSION_DECIMAL;
-
+const unsigned int rasqal_version_decimal = RASQAL_VERSION;
 
 
 /**
- * rasqal_init:
+ * rasqal_new_world:
  * 
- * Initialise the rasqal library.
+ * Allocate a new rasqal_world object.
  *
- * MUST be called before using any of the rasqal APIs.
+ * The rasqal_world is initialized with rasqal_world_open().
+ * Allocation and initialization are decoupled to allow
+ * changing settings on the world object before init.
+ *
+ * Return value: rasqal_world object or NULL on failure
  **/
-void
-rasqal_init(void) 
+rasqal_world*
+rasqal_new_world(void)
 {
-  if(rasqal_initialised || rasqal_initialising)
-    return;
-  rasqal_initialising=1;
-
-  raptor_init();
-
-  rasqal_uri_init();
-
-  /* last one declared is the default - RDQL */
-
-#ifdef RASQAL_QUERY_RDQL
-  rasqal_init_query_engine_rdql();
-#endif
-
-#ifdef RASQAL_QUERY_LAQRS
-  rasqal_init_query_engine_laqrs();
-#endif
-
-#ifdef RASQAL_QUERY_SPARQL  
-  rasqal_init_query_engine_sparql();
-#endif
-
-#ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
-  rasqal_raptor_init();
-#endif
-#ifdef RAPTOR_TRIPLES_SOURCE_REDLAND
-  rasqal_redland_init();
-#endif
-
-  rasqal_init_query_results();
-
-  rasqal_initialising=0;
-  rasqal_initialised=1;
-  rasqal_finishing=0;
+  return (rasqal_world*)RASQAL_CALLOC(rasqal_world, sizeof(rasqal_world), 1);
 }
 
 
 /**
- * rasqal_finish:
+ * rasqal_world_open:
+ * @world: rasqal_world object
+ * 
+ * Initialise the rasqal library.
  *
+ * Initializes a #rasqal_world object created by rasqal_new_world().
+ * Allocation and initialization are decoupled to allow
+ * changing settings on the world object before init.
+ * These settings include e.g. the raptor library instance set with
+ * rasqal_world_set_raptor().
+ *
+ * The initialized world object is used with subsequent rasqal API calls.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_world_open(rasqal_world *world)
+{
+  int rc;
+
+  if(!world)
+    return -1;
+    
+  if(world->opened++)
+    return 0; /* not an error */
+
+#ifdef RAPTOR_V2_AVAILABLE
+  /* Create and init a raptor_world unless one is provided externally with rasqal_world_set_raptor() */
+  if(!world->raptor_world_ptr) {
+    world->raptor_world_ptr = raptor_new_world();
+    if(!world->raptor_world_ptr)
+      return -1;
+    world->raptor_world_allocated_here = 1;
+    rc = raptor_world_open(world->raptor_world_ptr);
+    if(rc)
+      return rc;
+  }
+#else  
+  raptor_init();
+#endif
+
+  rc = rasqal_uri_init(world);
+  if(rc)
+    return rc;
+
+  rc = rasqal_xsd_init(world);
+  if(rc)
+    return rc;
+
+/* FIXME */
+#ifndef RAPTOR_ERROR_HANDLER_MAGIC
+#define RAPTOR_ERROR_HANDLER_MAGIC 0xD00DB1FF
+#endif
+  world->error_handlers.magic=RAPTOR_ERROR_HANDLER_MAGIC;
+
+  /* last one declared is the default - RDQL */
+
+#ifdef RASQAL_QUERY_RDQL
+  rc = rasqal_init_query_language_rdql(world);
+  if(rc)
+    return rc;
+#endif
+
+#ifdef RASQAL_QUERY_LAQRS
+  rc = rasqal_init_query_language_laqrs(world);
+  if(rc)
+    return rc;
+#endif
+
+#ifdef RASQAL_QUERY_SPARQL  
+  rc = rasqal_init_query_language_sparql(world);
+  if(rc)
+    return rc;
+#endif
+
+#ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
+  rc = rasqal_raptor_init(world);
+  if(rc)
+    return rc;
+#endif
+#ifdef RAPTOR_TRIPLES_SOURCE_REDLAND
+  rc = rasqal_redland_init(world);
+  if(rc)
+    return rc;
+#endif
+
+  rc = rasqal_init_query_results();
+  if(rc)
+    return rc;
+  
+  rc = rasqal_init_result_formats(world);
+  if(rc)
+    return rc;
+
+  return 0;
+}
+
+
+/**
+ * rasqal_free_world:
+ * @world: rasqal_world object
+ * 
  * Terminate the rasqal library.
  *
- * Must be called to clean up any resources used by the library.
+ * Destroys a rasqal_world object and all static information.
+ *
  **/
 void
-rasqal_finish(void) 
+rasqal_free_world(rasqal_world* world) 
 {
-  if(!rasqal_initialised || rasqal_finishing)
-    return;
-  rasqal_finishing=1;
-
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(world, rasqal_world);
+  
+  rasqal_finish_result_formats(world);
   rasqal_finish_query_results();
 
-  rasqal_delete_query_engine_factories();
+  rasqal_delete_query_language_factories(world);
 
 #ifdef RAPTOR_TRIPLES_SOURCE_REDLAND
   rasqal_redland_finish();
 #endif
 
-  rasqal_uri_finish();
+  rasqal_xsd_finish(world);
 
+  rasqal_uri_finish(world);
+
+#ifdef RAPTOR_V2_AVAILABLE
+  if(world->raptor_world_ptr && world->raptor_world_allocated_here)
+    raptor_free_world(world->raptor_world_ptr);
+#else
   raptor_finish();
+#endif
 
-  rasqal_initialising=0;
-  rasqal_initialised=0;
-  rasqal_finishing=0;
+  RASQAL_FREE(rasqal_world, world);
+}
+
+
+/**
+ * rasqal_world_set_raptor:
+ * @world: rasqal_world object
+ * @raptor_world_ptr: raptor_world object
+ * 
+ * Set the #raptor_world instance to be used with this #rasqal_world.
+ *
+ * If no raptor_world instance is set with this function,
+ * rasqal_world_open() creates a new instance.
+ *
+ * Ownership of the raptor_world is not taken. If the raptor library
+ * instance is set with this function, rasqal_free_world() will not
+ * free it.
+ *
+ **/
+void
+rasqal_world_set_raptor(rasqal_world* world, raptor_world* raptor_world_ptr)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(world, rasqal_world);
+  world->raptor_world_ptr = raptor_world_ptr;
+}
+
+
+/**
+ * rasqal_world_get_raptor:
+ * @world: rasqal_world object
+ * 
+ * Get the #raptor_world instance used by this #rasqal_world.
+ *
+ * Return value: raptor_world object or NULL on failure (e.g. not initialized)
+ **/
+raptor_world*
+rasqal_world_get_raptor(rasqal_world* world)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
+  return world->raptor_world_ptr;
 }
 
 
 /* helper functions */
 
-
 /*
- * rasqal_delete_query_engine_factories - helper function to delete all the registered query engine factories
+ * rasqal_free_query_language_factory - delete a query language factory
  */
 static void
-rasqal_delete_query_engine_factories(void)
+rasqal_free_query_language_factory(rasqal_query_language_factory *factory)
 {
-  rasqal_query_engine_factory *factory, *next;
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(factory, rasqal_query_language_factory);
   
-  for(factory=query_engines; factory; factory=next) {
-    next=factory->next;
-
+  if(factory) {
     if(factory->finish_factory)
       factory->finish_factory(factory);
 
-    RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->name);
-    RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->label);
+    if(factory->name)
+      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->name);
+    if(factory->label)
+      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->label);
     if(factory->alias)
-      RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->alias);
+      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->alias);
     if(factory->uri_string)
-      RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->uri_string);
+      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->uri_string);
 
-    RASQAL_FREE(rasqal_query_engine_factory, factory);
+    RASQAL_FREE(rasqal_query_language_factory, factory);
   }
-  query_engines=NULL;
+}
+
+
+/*
+ * rasqal_delete_query_language_factories - helper function to delete all the registered query language factories
+ */
+static void
+rasqal_delete_query_language_factories(rasqal_world *world)
+{
+  rasqal_query_language_factory *factory, *next;
+  
+  for(factory=world->query_languages; factory; factory=next) {
+    next=factory->next;
+    rasqal_free_query_language_factory(factory);
+  }
+  world->query_languages=NULL;
 }
 
 
 /* class methods */
 
 /*
- * rasqal_query_engine_register_factory - Register a syntax handled by a query factory
+ * rasqal_query_language_register_factory - Register a syntax handled by a query factory
  * @name: the short syntax name
  * @label: readable label for syntax
  * @uri_string: URI string of the syntax (or NULL)
@@ -225,14 +344,17 @@ rasqal_delete_query_engine_factories(void)
  * 
  * INTERNAL
  *
+ * Return value: non-0 on failure
  **/
-void
-rasqal_query_engine_register_factory(const char *name, const char *label,
+RASQAL_EXTERN_C
+int
+rasqal_query_language_register_factory(rasqal_world *world,
+                                     const char *name, const char *label,
                                      const char *alias,
                                      const unsigned char *uri_string,
-                                     void (*factory) (rasqal_query_engine_factory*)) 
+                                     void (*factory) (rasqal_query_language_factory*)) 
 {
-  rasqal_query_engine_factory *query, *h;
+  rasqal_query_language_factory *query, *h;
   char *name_copy, *label_copy, *alias_copy;
   unsigned char *uri_string_copy;
   
@@ -242,12 +364,12 @@ rasqal_query_engine_register_factory(const char *name, const char *label,
   RASQAL_DEBUG2("URI %s\n", (uri_string ? (const char*)uri_string : (const char*)"none"));
 #endif
   
-  query=(rasqal_query_engine_factory*)RASQAL_CALLOC(rasqal_query_engine_factory, 1,
-                                                    sizeof(rasqal_query_engine_factory));
+  query=(rasqal_query_language_factory*)RASQAL_CALLOC(rasqal_query_language_factory, 1,
+                                                    sizeof(rasqal_query_language_factory));
   if(!query)
-    RASQAL_FATAL1("Out of memory\n");
+    goto tidy_noquery;
 
-  for(h = query_engines; h; h = h->next ) {
+  for(h = world->query_languages; h; h = h->next ) {
     if(!strcmp(h->name, name) ||
        (alias && !strcmp(h->name, alias))) {
       RASQAL_FATAL2("query %s already registered\n", h->name);
@@ -255,37 +377,29 @@ rasqal_query_engine_register_factory(const char *name, const char *label,
   }
   
   name_copy=(char*)RASQAL_CALLOC(cstring, strlen(name)+1, 1);
-  if(!name_copy) {
-    RASQAL_FREE(rasqal_query, query);
-    RASQAL_FATAL1("Out of memory\n");
-  }
+  if(!name_copy)
+    goto tidy;
   strcpy(name_copy, name);
   query->name=name_copy;
         
   label_copy=(char*)RASQAL_CALLOC(cstring, strlen(label)+1, 1);
-  if(!label_copy) {
-    RASQAL_FREE(rasqal_query, query);
-    RASQAL_FATAL1("Out of memory\n");
-  }
+  if(!label_copy)
+    goto tidy;
   strcpy(label_copy, label);
   query->label=label_copy;
 
   if(uri_string) {
     uri_string_copy=(unsigned char*)RASQAL_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
-    if(!uri_string_copy) {
-    RASQAL_FREE(rasqal_query, query);
-    RASQAL_FATAL1("Out of memory\n");
-    }
+    if(!uri_string_copy)
+      goto tidy;
     strcpy((char*)uri_string_copy, (const char*)uri_string);
     query->uri_string=uri_string_copy;
   }
         
   if(alias) {
     alias_copy=(char*)RASQAL_CALLOC(cstring, strlen(alias)+1, 1);
-    if(!alias_copy) {
-      RASQAL_FREE(rasqal_query, query);
-      RASQAL_FATAL1("Out of memory\n");
-    }
+    if(!alias_copy)
+      goto tidy;
     strcpy(alias_copy, alias);
     query->alias=alias_copy;
   }
@@ -297,13 +411,22 @@ rasqal_query_engine_register_factory(const char *name, const char *label,
   RASQAL_DEBUG3("%s has context size %d\n", name, (int)query->context_length);
 #endif
   
-  query->next = query_engines;
-  query_engines = query;
+  query->next = world->query_languages;
+  world->query_languages = query;
+
+  return 0;
+
+  tidy:
+  rasqal_free_query_language_factory(query);
+  tidy_noquery:
+  rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_FATAL, NULL,
+                          "Out of memory in rasqal_query_language_register_factory()");
+  return 1;
 }
 
 
 /**
- * rasqal_get_query_engine_factory:
+ * rasqal_get_query_language_factory:
  * @name: the factory name or NULL for the default factory
  * @uri: the query syntax URI or NULL
  *
@@ -311,20 +434,24 @@ rasqal_query_engine_register_factory(const char *name, const char *label,
  * 
  * Return value: the factory object or NULL if there is no such factory
  **/
-rasqal_query_engine_factory*
-rasqal_get_query_engine_factory (const char *name, const unsigned char *uri)
+rasqal_query_language_factory*
+rasqal_get_query_language_factory(rasqal_world *world, const char *name,
+                                  const unsigned char *uri)
 {
-  rasqal_query_engine_factory *factory;
+  rasqal_query_language_factory *factory;
 
+  /* for compatibility with old API that does not call this - FIXME Remove V2 */
+  rasqal_world_open(world);
+  
   /* return 1st query if no particular one wanted - why? */
   if(!name && !uri) {
-    factory=query_engines;
+    factory=world->query_languages;
     if(!factory) {
-      RASQAL_DEBUG1("No (default) query_engines registered\n");
+      RASQAL_DEBUG1("No (default) query_languages registered\n");
       return NULL;
     }
   } else {
-    for(factory=query_engines; factory; factory=factory->next) {
+    for(factory=world->query_languages; factory; factory=factory->next) {
       if((name && !strcmp(factory->name, name)) ||
          (factory->alias && !strcmp(factory->alias, name)))
         break;
@@ -344,6 +471,7 @@ rasqal_get_query_engine_factory (const char *name, const unsigned char *uri)
 
 /**
  * rasqal_languages_enumerate:
+ * @world: rasqal_world object
  * @counter: index into the list of syntaxes
  * @name: pointer to store the name of the syntax (or NULL)
  * @label: pointer to store syntax readable label (or NULL)
@@ -354,13 +482,18 @@ rasqal_get_query_engine_factory (const char *name, const unsigned char *uri)
  * Return value: non 0 on failure of if counter is out of range
  **/
 int
-rasqal_languages_enumerate(const unsigned int counter,
+rasqal_languages_enumerate(rasqal_world *world,
+                           unsigned int counter,
                            const char **name, const char **label,
                            const unsigned char **uri_string)
 {
   unsigned int i;
-  rasqal_query_engine_factory *factory=query_engines;
+  rasqal_query_language_factory *factory;
 
+  /* for compatibility with old API that does not call this - FIXME Remove V2 */
+  rasqal_world_open(world);
+  
+  factory = world->query_languages;
   if(!factory)
     return 1;
 
@@ -382,6 +515,7 @@ rasqal_languages_enumerate(const unsigned int counter,
 
 /**
  * rasqal_language_name_check:
+ * @world: rasqal_world object
  * @name: the query language name
  *
  * Check name of a query language.
@@ -389,174 +523,128 @@ rasqal_languages_enumerate(const unsigned int counter,
  * Return value: non 0 if name is a known query language
  */
 int
-rasqal_language_name_check(const char *name) {
-  return (rasqal_get_query_engine_factory(name, NULL) != NULL);
+rasqal_language_name_check(rasqal_world* world, const char *name) {
+  return (rasqal_get_query_language_factory(world, name, NULL) != NULL);
 }
 
 
-/*
- * rasqal_query_fatal_error - Fatal Error from a query - Internal
- **/
+static const char* const rasqal_log_level_labels[RAPTOR_LOG_LEVEL_LAST+1]={
+  "none",
+  "fatal error",
+  "error",
+  "warning"
+};
+
+
+/* internal */
 void
-rasqal_query_fatal_error(rasqal_query* query, const char *message, ...)
+rasqal_log_error_simple(rasqal_world* world, raptor_log_level level,
+                        raptor_locator* locator, const char* message, ...)
 {
   va_list arguments;
 
-  va_start(arguments, message);
+  if(level == RAPTOR_LOG_LEVEL_NONE)
+    return;
 
-  rasqal_query_fatal_error_varargs(query, message, arguments);
-  
+  va_start(arguments, message);
+  rasqal_log_error_varargs(world, level, locator, message, arguments);
   va_end(arguments);
 }
 
 
-/*
- * rasqal_query_fatal_error_varargs - Fatal Error from a query - Internal
- **/
 void
-rasqal_query_fatal_error_varargs(rasqal_query* query, const char *message,
-                                 va_list arguments)
+rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
+                         raptor_locator* locator,
+                         const char* message, va_list arguments)
 {
-  query->failed=1;
+  char *buffer;
+  size_t length;
+  raptor_message_handler handler=world->error_handlers.handlers[level].handler;
+  void* handler_data=world->error_handlers.handlers[level].user_data;
+  
+  if(level == RAPTOR_LOG_LEVEL_NONE)
+    return;
 
-  if(query->fatal_error_handler) {
-    char *buffer=raptor_vsnprintf(message, arguments);
-    if(!buffer) {
-      fprintf(stderr, "rasqal_query_fatal_error_varargs: Out of memory\n");
-      return;
+  buffer=raptor_vsnprintf(message, arguments);
+  if(!buffer) {
+    if(locator) {
+#ifdef RAPTOR_V2_AVAILABLE
+      raptor_print_locator_v2(world->raptor_world_ptr, stderr, locator);
+#else
+      raptor_print_locator(stderr, locator);
+#endif
+      fputc(' ', stderr);
     }
-
-    query->fatal_error_handler(query->fatal_error_user_data, 
-                               &query->locator, buffer); 
-    RASQAL_FREE(cstring, buffer);
-    abort();
+    fputs("rasqal ", stderr);
+    fputs(rasqal_log_level_labels[level], stderr);
+    fputs(" - ", stderr);
+    vfprintf(stderr, message, arguments);
+    fputc('\n', stderr);
+    return;
   }
 
-  raptor_print_locator(stderr, &query->locator);
-  fprintf(stderr, " rasqal fatal error - ");
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-
-  abort();
-}
-
-
-/*
- * rasqal_query_error - Error from a query - Internal
- **/
-void
-rasqal_query_error(rasqal_query* query, const char *message, ...)
-{
-  va_list arguments;
-
-  va_start(arguments, message);
-
-  rasqal_query_error_varargs(query, message, arguments);
+  length=strlen(buffer);
+  if(buffer[length-1]=='\n')
+    buffer[length-1]='\0';
   
-  va_end(arguments);
+  if(handler)
+    /* This is the single place in rasqal that the user error handler
+     * functions are called.
+     */
+    handler(handler_data, locator, buffer);
+  else {
+    if(locator) {
+#ifdef RAPTOR_V2_AVAILABLE
+      raptor_print_locator_v2(world->raptor_world_ptr, stderr, locator);
+#else
+      raptor_print_locator(stderr, locator);
+#endif
+      fputc(' ', stderr);
+    }
+    fputs("rasqal ", stderr);
+    fputs(rasqal_log_level_labels[level], stderr);
+    fputs(" - ", stderr);
+    fputs(buffer, stderr);
+    fputc('\n', stderr);
+  }
+
+  RASQAL_FREE(cstring, buffer);
 }
 
 
 /*
  * rasqal_query_simple_error - Error from a query - Internal
  *
- * Matches the rasqal_simple_message_handler API but same as
+ * Matches the raptor_simple_message_handler API but same as
  * rasqal_query_error 
- **/
+ */
 void
-rasqal_query_simple_error(void* query, const char *message, ...)
+rasqal_query_simple_error(void* user_data, const char *message, ...)
 {
+  rasqal_query* query=(rasqal_query*)user_data;
+
   va_list arguments;
 
   va_start(arguments, message);
 
-  rasqal_query_error_varargs((rasqal_query*)query, message, arguments);
+  query->failed=1;
+  rasqal_log_error_varargs(query->world,
+                           RAPTOR_LOG_LEVEL_ERROR, NULL,
+                           message, arguments);
   
   va_end(arguments);
 }
 
 
-/*
- * rasqal_query_error_varargs - Error from a query - Internal
- **/
-void
-rasqal_query_error_varargs(rasqal_query* query, const char *message, 
-                           va_list arguments)
-{
-  query->failed=1;
-
-  if(query->error_handler) {
-    char *buffer=raptor_vsnprintf(message, arguments);
-    if(!buffer) {
-      fprintf(stderr, "rasqal_query_error_varargs: Out of memory\n");
-      return;
-    }
-    query->error_handler(query->error_user_data, 
-                         &query->locator, buffer);
-    RASQAL_FREE(cstring, buffer);
-    return;
-  }
-
-  raptor_print_locator(stderr, &query->locator);
-  fprintf(stderr, " rasqal error - ");
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-}
-
-
-/*
- * rasqal_query_warning - Warning from a query - Internal
- **/
-void
-rasqal_query_warning(rasqal_query* query, const char *message, ...)
-{
-  va_list arguments;
-
-  va_start(arguments, message);
-
-  rasqal_query_warning_varargs(query, message, arguments);
-
-  va_end(arguments);
-}
-
-
-/*
- * rasqal_query_warning - Warning from a query - Internal
- **/
-void
-rasqal_query_warning_varargs(rasqal_query* query, const char *message, 
-                             va_list arguments)
-{
-
-  if(query->warning_handler) {
-    char *buffer=raptor_vsnprintf(message, arguments);
-    if(!buffer) {
-      fprintf(stderr, "rasqal_query_warning_varargs: Out of memory\n");
-      return;
-    }
-    query->warning_handler(query->warning_user_data,
-                           &query->locator, buffer);
-    RASQAL_FREE(cstring, buffer);
-    return;
-  }
-
-  raptor_print_locator(stderr, &query->locator);
-  fprintf(stderr, " rasqal warning - ");
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-}
-
-
-
 /* wrapper */
 const char*
 rasqal_basename(const char *name)
 {
-  char *p;
-  if((p=strrchr(name, '/')))
-    name=p+1;
-  else if((p=strrchr(name, '\\')))
-    name=p+1;
+  const char *p;
+  if((p = strrchr(name, '/')))
+    name = p+1;
+  else if((p = strrchr(name, '\\')))
+    name = p+1;
 
   return name;
 }
@@ -687,78 +775,81 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 }
 
 
-raptor_uri* rasqal_xsd_namespace_uri=NULL;
-
-raptor_uri* rasqal_xsd_integer_uri=NULL;
-raptor_uri* rasqal_xsd_double_uri=NULL;
-raptor_uri* rasqal_xsd_float_uri=NULL;
-raptor_uri* rasqal_xsd_boolean_uri=NULL;
-raptor_uri* rasqal_xsd_decimal_uri=NULL;
-raptor_uri* rasqal_xsd_datetime_uri=NULL;
-raptor_uri* rasqal_xsd_string_uri=NULL;
-
-raptor_uri* rasqal_rdf_namespace_uri=NULL;
-
-raptor_uri* rasqal_rdf_first_uri=NULL;
-raptor_uri* rasqal_rdf_rest_uri=NULL;
-raptor_uri* rasqal_rdf_nil_uri=NULL;
-
-
-void
-rasqal_uri_init(void) 
+int
+rasqal_uri_init(rasqal_world* world) 
 {
-  rasqal_xsd_namespace_uri=raptor_new_uri(raptor_xmlschema_datatypes_namespace_uri);
-  
-  rasqal_xsd_integer_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"integer");
-  rasqal_xsd_double_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"double");
-  rasqal_xsd_float_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"float");
-  rasqal_xsd_boolean_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"boolean");
-  rasqal_xsd_decimal_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"decimal");
-  rasqal_xsd_datetime_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"dateTime");
-  rasqal_xsd_string_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"string");
+#ifdef RAPTOR_V2_AVAILABLE
+  world->rdf_namespace_uri = raptor_new_uri_v2(world->raptor_world_ptr, raptor_rdf_namespace_uri);
+#else
+  world->rdf_namespace_uri = raptor_new_uri(raptor_rdf_namespace_uri);
+#endif
+  if(!world->rdf_namespace_uri)
+    goto oom;
 
-  rasqal_rdf_namespace_uri=raptor_new_uri(raptor_rdf_namespace_uri);
+#ifdef RAPTOR_V2_AVAILABLE
+  world->rdf_first_uri = raptor_new_uri_from_uri_local_name_v2(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"first");
+  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name_v2(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"rest");
+  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name_v2(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"nil");
+#else
+  world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"first");
+  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"rest");
+  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"nil");
+#endif
 
-  rasqal_rdf_first_uri=raptor_new_uri_from_uri_local_name(rasqal_rdf_namespace_uri, (const unsigned char*)"first");
-  rasqal_rdf_rest_uri=raptor_new_uri_from_uri_local_name(rasqal_rdf_namespace_uri, (const unsigned char*)"rest");
-  rasqal_rdf_nil_uri=raptor_new_uri_from_uri_local_name(rasqal_rdf_namespace_uri, (const unsigned char*)"nil");
-  
+  if(!world->rdf_first_uri || !world->rdf_rest_uri || !world->rdf_nil_uri)
+    goto oom;
+
+  return 0;
+
+  oom:
+  rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_FATAL,
+                          NULL,
+                          "Out of memory in rasqal_uri_init()");
+  return 1;
 }
 
 
 void
-rasqal_uri_finish(void) 
+rasqal_uri_finish(rasqal_world* world) 
 {
-  if(rasqal_xsd_integer_uri)
-    raptor_free_uri(rasqal_xsd_integer_uri);
-  if(rasqal_xsd_double_uri)
-    raptor_free_uri(rasqal_xsd_double_uri);
-  if(rasqal_xsd_float_uri)
-    raptor_free_uri(rasqal_xsd_float_uri);
-  if(rasqal_xsd_boolean_uri)
-    raptor_free_uri(rasqal_xsd_boolean_uri);
-  if(rasqal_xsd_decimal_uri)
-    raptor_free_uri(rasqal_xsd_decimal_uri);
-  if(rasqal_xsd_datetime_uri)
-    raptor_free_uri(rasqal_xsd_datetime_uri);
-  if(rasqal_xsd_string_uri)
-    raptor_free_uri(rasqal_xsd_string_uri);
-  if(rasqal_xsd_namespace_uri)
-    raptor_free_uri(rasqal_xsd_namespace_uri);
-
-  if(rasqal_rdf_first_uri)
-    raptor_free_uri(rasqal_rdf_first_uri);
-  if(rasqal_rdf_rest_uri)
-    raptor_free_uri(rasqal_rdf_rest_uri);
-  if(rasqal_rdf_nil_uri)
-    raptor_free_uri(rasqal_rdf_nil_uri);
-  if(rasqal_rdf_namespace_uri)
-    raptor_free_uri(rasqal_rdf_namespace_uri);
-
+#ifdef RAPTOR_V2_AVAILABLE
+  if(world->rdf_first_uri) {
+    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_first_uri);
+    world->rdf_first_uri = NULL;
+  }
+  if(world->rdf_rest_uri) {
+    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_rest_uri);
+    world->rdf_rest_uri = NULL;
+  }
+  if(world->rdf_nil_uri) {
+    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_nil_uri);
+    world->rdf_nil_uri = NULL;
+  }
+  if(world->rdf_namespace_uri) {
+    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_namespace_uri);
+    world->rdf_namespace_uri = NULL;
+  }
+#else
+  if(world->rdf_first_uri) {
+    raptor_free_uri(world->rdf_first_uri);
+    world->rdf_first_uri=NULL;
+  }
+  if(world->rdf_rest_uri) {
+    raptor_free_uri(world->rdf_rest_uri);
+    world->rdf_rest_uri=NULL;
+  }
+  if(world->rdf_nil_uri) {
+    raptor_free_uri(world->rdf_nil_uri);
+    world->rdf_nil_uri=NULL;
+  }
+  if(world->rdf_namespace_uri) {
+    raptor_free_uri(world->rdf_namespace_uri);
+    world->rdf_namespace_uri=NULL;
+  }
+#endif
 }
 
 
-
 /**
  * rasqal_query_set_default_generate_bnodeid_parameters - Set default bnodeid generation parameters
  * @rdf_query: #rasqal_query object
@@ -905,6 +996,8 @@ rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
 void
 rasqal_free_memory(void *ptr)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(ptr, memory);
+  
   RASQAL_FREE(void, ptr);
 }
 
