@@ -2,9 +2,7 @@
  *
  * roqet.c - Rasqal RDF Query test program
  *
- * $Id: roqet.c 12098 2007-04-22 07:31:28Z dajobe $
- *
- * Copyright (C) 2004-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -45,7 +43,8 @@
 #endif
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#else
+#endif
+#ifndef HAVE_GETOPT
 #include <rasqal_getopt.h>
 #endif
 #ifdef HAVE_STDLIB_H
@@ -57,41 +56,47 @@
 
 /* Rasqal includes */
 #include <rasqal.h>
+#ifdef RASQAL_INTERNAL
+#include <rasqal_internal.h>
+#endif
+
 
 #ifdef NEED_OPTIND_DECLARATION
 extern int optind;
 extern char *optarg;
 #endif
 
-int rdql_parser_error(const char *msg);
 int main(int argc, char *argv[]);
 
 
 static char *program=NULL;
 
 
-int
-rdql_parser_error(const char *msg) 
-{
-  fprintf(stderr, "%s: Query parsing error: %s\n", program, msg);
-  return (0);
-}
-
-
 #ifdef HAVE_GETOPT_LONG
 #define HELP_TEXT(short, long, description) "  -" short ", --" long "  " description
+#define HELP_TEXT_LONG(long, description) "      --" long "  " description
 #define HELP_ARG(short, long) "--" #long
-#define HELP_PAD "\n                           "
+#define HELP_PAD "\n                            "
 #else
 #define HELP_TEXT(short, long, description) "  -" short "  " description
+#define HELP_TEXT_LONG(long, description)
 #define HELP_ARG(short, long) "-" #short
 #define HELP_PAD "\n      "
 #endif
 
-
-#define GETOPT_STRING "cd:e:f:hi:nr:qs:vw"
+#ifdef RASQAL_INTERNAL
+/* add 'g:' */
+#define GETOPT_STRING "cd:D:e:f:g:G:hi:nr:qs:vw"
+#else
+#define GETOPT_STRING "cd:D:e:f:G:hi:nr:qs:vw"
+#endif
 
 #ifdef HAVE_GETOPT_LONG
+
+#ifdef RASQAL_INTERNAL
+#define STORE_RESULTS_FLAG 0x100
+#endif
+
 static struct option long_options[] =
 {
   /* name, has_arg, flag, val */
@@ -100,13 +105,21 @@ static struct option long_options[] =
   {"dryrun", 0, 0, 'n'},
   {"exec", 1, 0, 'e'},
   {"feature", 1, 0, 'f'},
+#ifdef RASQAL_INTERNAL
+  {"engine", 1, 0, 'g'},
+#endif
   {"help", 0, 0, 'h'},
   {"input", 1, 0, 'i'},
   {"quiet", 0, 0, 'q'},
   {"results", 1, 0, 'r'},
   {"source", 1, 0, 's'},
+  {"data", 1, 0, 'D'},
+  {"named", 1, 0, 'G'},
   {"version", 0, 0, 'v'},
   {"walk-query", 0, 0, 'w'},
+#ifdef STORE_RESULTS_FLAG
+  {"store-results", 1, 0, STORE_RESULTS_FLAG},
+#endif
   {NULL, 0, 0, 0}
 };
 #endif
@@ -129,8 +142,16 @@ static void
 roqet_error_handler(void *user_data, 
                     raptor_locator* locator, const char *message) 
 {
+#ifdef RAPTOR_V2_AVAILABLE
+  rasqal_world* world = (rasqal_world *)user_data;
+#endif
+
   fprintf(stderr, "%s: Error - ", program);
+#ifdef RAPTOR_V2_AVAILABLE
+  raptor_print_locator_v2(rasqal_world_get_raptor(world), stderr, locator);
+#else
   raptor_print_locator(stderr, locator);
+#endif
   fprintf(stderr, " - %s\n", message);
 
   error_count++;
@@ -159,6 +180,7 @@ roqet_graph_pattern_walk(rasqal_graph_pattern *gp, int gp_index,
   int seen;
   raptor_sequence *seq;
   int idx;
+  rasqal_expression* expr;
   
   op=rasqal_graph_pattern_get_operator(gp);
   
@@ -177,7 +199,9 @@ roqet_graph_pattern_walk(rasqal_graph_pattern *gp, int gp_index,
   /* look for triples */
   seen=0;
   while(1) {
-    rasqal_triple* t=rasqal_graph_pattern_get_triple(gp, triple_index);
+    rasqal_triple* t;
+
+    t = rasqal_graph_pattern_get_triple(gp, triple_index);
     if(!t)
       break;
     
@@ -207,7 +231,8 @@ roqet_graph_pattern_walk(rasqal_graph_pattern *gp, int gp_index,
 
     gp_index=0;
     while(1) {
-      rasqal_graph_pattern* sgp=rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+      rasqal_graph_pattern* sgp;
+      sgp=rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
       if(!sgp)
         break;
       
@@ -220,27 +245,12 @@ roqet_graph_pattern_walk(rasqal_graph_pattern *gp, int gp_index,
   }
   
 
-  /* look for constraints */
-  seq=rasqal_graph_pattern_get_constraint_sequence(gp);
-  if(seq && raptor_sequence_size(seq) > 0) {
-    roqet_write_indent(fh, indent);
-    fprintf(fh, "constraints (%d) {\n", raptor_sequence_size(seq));
-
-    gp_index=0;
-    while(1) {
-      rasqal_expression* expr=rasqal_graph_pattern_get_constraint(gp, gp_index);
-      if(!expr)
-        break;
-      
-      roqet_write_indent(fh, indent+2);
-      fprintf(fh, "constraint #%d { ", gp_index);
-      rasqal_expression_print(expr, fh);
-      fputs("}\n", fh);
-      
-      gp_index++;
-    }
-
+  /* look for filter */
+  expr = rasqal_graph_pattern_get_filter_expression(gp);
+  if(expr) {
     roqet_write_indent(fh, indent);
+    fputs("filter { ", fh);
+    rasqal_expression_print(expr, fh);
     fputs("}\n", fh);
   }
   
@@ -382,7 +392,8 @@ main(int argc, char *argv[])
   int quiet=0;
   int count=0;
   int dryrun=0;
-  raptor_sequence* source_uris=NULL;
+  raptor_sequence* data_source_uris=NULL;
+  raptor_sequence* named_source_uris=NULL;
   raptor_serializer* serializer=NULL;
   const char *serializer_syntax_name="ntriples";
   query_output_format output_format= QUERY_OUTPUT_UNKNOWN;
@@ -390,6 +401,14 @@ main(int argc, char *argv[])
   rasqal_feature query_feature=(rasqal_feature)-1;
   int query_feature_value= -1;
   unsigned char* query_feature_string_value=NULL;
+  rasqal_world *world;
+#ifdef RAPTOR_V2_AVAILABLE
+  raptor_world* raptor_world_ptr;
+#endif
+#ifdef RASQAL_INTERNAL
+  int store_results= -1;
+  const rasqal_query_execution_factory* engine = NULL;
+#endif
   
   program=argv[0];
   if((p=strrchr(program, '/')))
@@ -397,14 +416,31 @@ main(int argc, char *argv[])
   else if((p=strrchr(program, '\\')))
     program=p+1;
   argv[0]=program;
-  
 
-  rasqal_init();
+  world=rasqal_new_world();
+  if(!world || rasqal_world_open(world)) {
+    fprintf(stderr, "%s: rasqal_world init failed\n", program);
+    return(1);
+  }
+  
+#ifdef RAPTOR_V2_AVAILABLE
+  raptor_world_ptr = rasqal_world_get_raptor(world);
+#endif
+  
+#ifdef STORE_RESULTS_FLAG
+  /* This is for debugging only */
+  if(1) {
+    char* sr=getenv("RASQAL_DEBUG_STORE_RESULTS");
+    if(sr)
+      store_results=atoi(sr);
+  }
+#endif
 
   while (!usage && !help)
   {
     int c;
     raptor_uri *source_uri;
+    raptor_sequence** seq_p=NULL;
     
 #ifdef HAVE_GETOPT_LONG
     int option_index = 0;
@@ -468,7 +504,7 @@ main(int argc, char *argv[])
             for(i=0; i < (int)rasqal_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
-              if(!rasqal_features_enumerate((rasqal_feature)i, &feature_name, NULL, &feature_label)) {
+              if(!rasqal_features_enumerate(world, (rasqal_feature)i, &feature_name, NULL, &feature_label)) {
                 const char *feature_type=(rasqal_feature_value_type((rasqal_feature)i) == 0) ? "" : " (string)";
                 fprintf(stderr, "  %-20s  %s%s\n", feature_name, feature_label, 
                        feature_type);
@@ -476,7 +512,7 @@ main(int argc, char *argv[])
             }
             fputs("Features are set with `" HELP_ARG(f, feature) " FEATURE=VALUE or `-f FEATURE'\nand take a decimal integer VALUE except where noted, defaulting to 1 if omitted.\n", stderr);
 
-            rasqal_finish();
+            rasqal_free_world(world);
             exit(0);
           } else {
             int i;
@@ -486,7 +522,7 @@ main(int argc, char *argv[])
               const char *feature_name;
               size_t len;
               
-              if(rasqal_features_enumerate((rasqal_feature)i, &feature_name, NULL, NULL))
+              if(rasqal_features_enumerate(world, (rasqal_feature)i, &feature_name, NULL, NULL))
                 continue;
               len=strlen(feature_name);
               if(!strncmp(optarg, feature_name, len)) {
@@ -531,9 +567,15 @@ main(int argc, char *argv[])
           if(!strcmp(optarg, "simple"))
             results_formatter=NULL;
           else {
-            results_formatter=rasqal_new_query_results_formatter(optarg, NULL);
+            results_formatter=rasqal_new_query_results_formatter(world, optarg, NULL);
             if(!results_formatter) {
-              if(raptor_serializer_syntax_name_check(optarg))
+              if(
+#ifdef RAPTOR_V2_AVAILABLE
+                raptor_serializer_syntax_name_check_v2(raptor_world_ptr, optarg)
+#else
+                raptor_serializer_syntax_name_check(optarg)
+#endif
+                )
                 serializer_syntax_name=optarg;
               else {
                 fprintf(stderr, 
@@ -548,7 +590,7 @@ main(int argc, char *argv[])
         break;
 
       case 'i':
-	if(rasqal_language_name_check(optarg))
+	if(rasqal_language_name_check(world, optarg))
           ql_name=optarg;
 	else {
           int i;
@@ -559,7 +601,7 @@ main(int argc, char *argv[])
           for(i=0; 1; i++) {
             const char *help_name;
             const char *help_label;
-            if(rasqal_languages_enumerate(i, &help_name, &help_label, NULL))
+            if(rasqal_languages_enumerate(world, i, &help_name, &help_label, NULL))
               break;
             fprintf(stderr, "  %-12s for %s\n", help_name, help_label);
           }
@@ -572,34 +614,53 @@ main(int argc, char *argv[])
         break;
 
       case 's':
-        if(!source_uris) {
-          source_uris=raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri,
-                                          (raptor_sequence_print_handler*)raptor_sequence_print_uri);
-          if(!source_uris) {
-            fprintf(stderr, "%s: Failed to create source sequence\n", program);
-          return(1);
-          }
-        }
-        
+      case 'D':
+      case 'G':
         if(!access((const char*)optarg, R_OK)) {
           unsigned char* source_uri_string=raptor_uri_filename_to_uri_string((const char*)optarg);
-          source_uri=raptor_new_uri(source_uri_string);
+#ifdef RAPTOR_V2_AVAILABLE
+          source_uri = raptor_new_uri_v2(raptor_world_ptr, source_uri_string);
+#else
+          source_uri = raptor_new_uri(source_uri_string);
+#endif
           raptor_free_memory(source_uri_string);
         } else
-          source_uri=raptor_new_uri((const unsigned char*)optarg);
+#ifdef RAPTOR_V2_AVAILABLE
+          source_uri = raptor_new_uri_v2(raptor_world_ptr, (const unsigned char*)optarg);
+#else
+          source_uri = raptor_new_uri((const unsigned char*)optarg);
+#endif
 
         if(!source_uri) {
           fprintf(stderr, "%s: Failed to create source URI for %s\n",
                   program, optarg);
           return(1);
         }
-        raptor_sequence_push(source_uris, source_uri);
+
+        seq_p= (c=='D') ? &data_source_uris : &named_source_uris;
+        if(!*seq_p) {
+#ifdef RAPTOR_V2_AVAILABLE
+          *seq_p = raptor_new_sequence_v2((raptor_sequence_free_handler_v2*)raptor_free_uri_v2,
+                                          (raptor_sequence_print_handler_v2*)raptor_uri_print_v2,
+                                          (void*)raptor_world_ptr);
+
+#else
+          *seq_p = raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri,
+                                       (raptor_sequence_print_handler*)raptor_uri_print);
+#endif
+          if(!*seq_p) {
+            fprintf(stderr, "%s: Failed to create source sequence\n", program);
+            return(1);
+          }
+        }
+
+        raptor_sequence_push(*seq_p, source_uri);
         break;
 
       case 'v':
         fputs(rasqal_version_string, stdout);
         fputc('\n', stdout);
-        rasqal_finish();
+        rasqal_free_world(world);
         exit(0);
 
       case 'w':
@@ -609,6 +670,18 @@ main(int argc, char *argv[])
         output_format=QUERY_OUTPUT_STRUCTURE;
         break;
 
+#ifdef STORE_RESULTS_FLAG
+      case STORE_RESULTS_FLAG:
+        store_results=(!strcmp(optarg, "yes") || !strcmp(optarg, "YES"));
+        break;
+#endif
+
+#ifdef RASQAL_INTERNAL
+      case 'g':
+        engine = rasqal_query_get_engine_by_name(optarg);
+        break;
+#endif
+
     }
     
   }
@@ -637,7 +710,7 @@ main(int argc, char *argv[])
     }
     fprintf(stderr, "Try `%s " HELP_ARG(h, help) "' for more information.\n",
                     program);
-    rasqal_finish();
+    rasqal_free_world(world);
 
     exit(1);
   }
@@ -664,7 +737,7 @@ main(int argc, char *argv[])
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
-      if(rasqal_languages_enumerate(i, &help_name, &help_label, NULL))
+      if(rasqal_languages_enumerate(world, i, &help_name, &help_label, NULL))
         break;
       printf("    %-15s         %s", help_name, help_label);
       if(!i)
@@ -672,21 +745,29 @@ main(int argc, char *argv[])
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("r", "results FORMAT  ", "Set query results format to one of:"));
+    puts(HELP_TEXT("r", "results FORMAT  ", "Set query results output format to one of:"));
     puts("    For variable bindings and boolean results:");
     puts("      simple                A simple text format (default)");
     for(i=0; i < (int)raptor_get_feature_count(); i++) {
       const char *name;
       const char *label;
-      if(!rasqal_query_results_formats_enumerate(i, &name, &label, 
-                                                 NULL))
+      int qr_flags=0;
+      if(!rasqal_query_results_formats_enumerate(world, i, &name, &label, 
+                                                 NULL, NULL, &qr_flags) &&
+         (qr_flags & RASQAL_QUERY_RESULTS_FORMAT_FLAG_WRITER))
         printf("      %-10s            %s\n", name, label);
     }
     puts("    For RDF graph results:");
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
-      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+      if(
+#ifdef RAPTOR_V2_AVAILABLE
+         raptor_serializers_enumerate_v2(raptor_world_ptr, i, &help_name, &help_label, NULL, NULL)
+#else
+         raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL)
+#endif
+         )
         break;
       printf("      %-15s       %s", help_name, help_label);
       if(!i)
@@ -695,21 +776,29 @@ main(int argc, char *argv[])
         putchar('\n');
     }
     puts("\nAdditional options:");
-    puts(HELP_TEXT("c", "count            ", "Count triples - no output"));
-    puts(HELP_TEXT("d", "dump-query FORMAT", "Print the parsed query out in FORMAT:"));
+    puts(HELP_TEXT("c", "count             ", "Count triples - no output"));
+    puts(HELP_TEXT("d", "dump-query FORMAT ", "Print the parsed query out in FORMAT:"));
     for(i=1; i <= QUERY_OUTPUT_LAST; i++)
-      printf("      %-15s        %s\n", query_output_format_labels[i][0],
+      printf("      %-15s         %s\n", query_output_format_labels[i][0],
              query_output_format_labels[i][1]);
     puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set query features" HELP_PAD "Use `-f help' for a list of valid features"));
-    puts(HELP_TEXT("h", "help             ", "Print this help, then exit"));
-    puts(HELP_TEXT("n", "dryrun           ", "Prepare but do not run the query"));
-    puts(HELP_TEXT("q", "quiet            ", "No extra information messages"));
-    puts(HELP_TEXT("s", "source URI       ", "Query against RDF data at source URI"));
-    puts(HELP_TEXT("v", "version          ", "Print the Rasqal version"));
-    puts(HELP_TEXT("w", "walk-query       ", "Print query.  Same as '-d structure'"));
+#ifdef RASQAL_INTERNAL
+    puts(HELP_TEXT("g", "engine NAME       ", "INTERNAL: Pick execution engine NAME"));
+#endif
+    puts(HELP_TEXT("h", "help              ", "Print this help, then exit"));
+    puts(HELP_TEXT("n", "dryrun            ", "Prepare but do not run the query"));
+    puts(HELP_TEXT("q", "quiet             ", "No extra information messages"));
+    puts(HELP_TEXT("D", "data URI          ", "RDF data source URI"));
+    puts(HELP_TEXT("G", "named URI         ", "RDF named graph data source URI"));
+    puts(HELP_TEXT("s", "source URI        ", "Same as `-G URI'"));
+    puts(HELP_TEXT("v", "version           ", "Print the Rasqal version"));
+    puts(HELP_TEXT("w", "walk-query        ", "Print query.  Same as '-d structure'"));
+#ifdef STORE_RESULTS_FLAG
+    puts(HELP_TEXT_LONG("store-results BOOL", "DEBUG: Set store results yes/no BOOL"));
+#endif
     puts("\nReport bugs to http://bugs.librdf.org/");
 
-    rasqal_finish();
+    rasqal_free_world(world);
     
     exit(0);
   }
@@ -741,7 +830,11 @@ main(int argc, char *argv[])
     }
     
     if(uri_string) {
-      uri=raptor_new_uri(uri_string);
+#ifdef RAPTOR_V2_AVAILABLE
+      uri = raptor_new_uri_v2(raptor_world_ptr, uri_string);
+#else
+      uri = raptor_new_uri(uri_string);
+#endif
       if(!uri) {
         fprintf(stderr, "%s: Failed to create URI for %s\n",
                 program, uri_string);
@@ -752,18 +845,26 @@ main(int argc, char *argv[])
   }
   
 
+#ifdef RAPTOR_V2_AVAILABLE
+  if(!base_uri_string) {
+    if(uri)
+      base_uri=raptor_uri_copy_v2(raptor_world_ptr, uri);
+  } else
+    base_uri=raptor_new_uri_v2(raptor_world_ptr, base_uri_string);
+#else
   if(!base_uri_string) {
     if(uri)
       base_uri=raptor_uri_copy(uri);
-  } else {
+  } else
     base_uri=raptor_new_uri(base_uri_string);
-    if(!base_uri) {
-      fprintf(stderr, "%s: Failed to create URI for %s\n",
-              program, base_uri_string);
-      return(1);
-    }
+#endif
+  if(base_uri_string && !base_uri) {
+    fprintf(stderr, "%s: Failed to create URI for %s\n",
+            program, base_uri_string);
+    return(1);
   }
 
+
   if(query_string) {
     /* NOP - already got it */
   } else if(!uri_string) {
@@ -812,9 +913,14 @@ main(int argc, char *argv[])
     raptor_free_stringbuffer(sb);
     query_from_string=0;
   } else {
-    raptor_www *www=raptor_www_new();
+    raptor_www *www;
+#ifdef RAPTOR_V2_AVAILABLE
+    www = raptor_www_new_v2(raptor_world_ptr);
+#else
+    www = raptor_www_new();
+#endif
     if(www) {
-      raptor_www_set_error_handler(www, roqet_error_handler, NULL);
+      raptor_www_set_error_handler(www, roqet_error_handler, world);
       raptor_www_fetch_to_string(www, uri, &query_string, NULL, malloc);
       raptor_www_free(www);
     }
@@ -851,25 +957,59 @@ main(int argc, char *argv[])
     }
   }
   
-  rq=rasqal_new_query((const char*)ql_name, (const unsigned char*)ql_uri);
-  rasqal_query_set_error_handler(rq, NULL, roqet_error_handler);
-  rasqal_query_set_fatal_error_handler(rq, NULL, roqet_error_handler);
+  rq=rasqal_new_query(world, (const char*)ql_name, (const unsigned char*)ql_uri);
+  rasqal_query_set_error_handler(rq, world, roqet_error_handler);
+  rasqal_query_set_fatal_error_handler(rq, world, roqet_error_handler);
 
   if(query_feature_value >= 0)
     rasqal_query_set_feature(rq, query_feature, query_feature_value);
   if(query_feature_string_value)
     rasqal_query_set_feature_string(rq, query_feature,
                                     query_feature_string_value);
+
+#ifdef STORE_RESULTS_FLAG
+  if(store_results >= 0)
+    rasqal_query_set_store_results(rq, store_results);
+#endif
   
-  if(source_uris) {
-    while(raptor_sequence_size(source_uris)) {
-      raptor_uri* source_uri=(raptor_uri*)raptor_sequence_pop(source_uris);
+  if(named_source_uris) {
+    while(raptor_sequence_size(named_source_uris)) {
+      raptor_uri* source_uri=(raptor_uri*)raptor_sequence_pop(named_source_uris);
       if(rasqal_query_add_data_graph(rq, source_uri, source_uri, 
                                      RASQAL_DATA_GRAPH_NAMED)) {
         fprintf(stderr, "%s: Failed to add named graph %s\n", program, 
-                raptor_uri_as_string(source_uri));
+#ifdef RAPTOR_V2_AVAILABLE
+                raptor_uri_as_string_v2(raptor_world_ptr, source_uri)
+#else
+                raptor_uri_as_string(source_uri)
+#endif
+                );
       }
+#ifdef RAPTOR_V2_AVAILABLE
+      raptor_free_uri_v2(raptor_world_ptr, source_uri);
+#else
       raptor_free_uri(source_uri);
+#endif
+    }
+  }
+  if(data_source_uris) {
+    while(raptor_sequence_size(data_source_uris)) {
+      raptor_uri* source_uri=(raptor_uri*)raptor_sequence_pop(data_source_uris);
+      if(rasqal_query_add_data_graph(rq, source_uri, NULL,
+                                     RASQAL_DATA_GRAPH_BACKGROUND)) {
+        fprintf(stderr, "%s: Failed to add to default graph %s\n", program, 
+#ifdef RAPTOR_V2_AVAILABLE
+                raptor_uri_as_string_v2(raptor_world_ptr, source_uri)
+#else
+                raptor_uri_as_string(source_uri)
+#endif
+                );
+      }
+#ifdef RAPTOR_V2_AVAILABLE
+      raptor_free_uri_v2(raptor_world_ptr, source_uri);
+#else
+      raptor_free_uri(source_uri);
+#endif
     }
   }
 
@@ -919,7 +1059,13 @@ main(int argc, char *argv[])
   if(dryrun)
     goto tidy_query;
 
-  if(!(results=rasqal_query_execute(rq))) {
+#ifdef RASQAL_INTERNAL
+  results = rasqal_query_execute_with_engine(rq, engine);
+#else
+  results = rasqal_query_execute(rq);
+#endif
+
+  if(!results) {
     fprintf(stderr, "%s: Query execution failed\n", program);
     rc=1;
     goto tidy_query;
@@ -935,9 +1081,12 @@ main(int argc, char *argv[])
       goto tidy_query;
     }
 
-    rasqal_query_results_formatter_write(iostr, results_formatter,
-                                         results, base_uri);
+    rc = rasqal_query_results_formatter_write(iostr, results_formatter,
+                                              results, base_uri);
     raptor_free_iostream(iostr);
+    if(rc)
+      fprintf(stderr, "%s: Formatting results failed\n", program);
+    
   } else {
     if(rasqal_query_results_is_bindings(results)) {
       if(!quiet)
@@ -982,7 +1131,11 @@ main(int argc, char *argv[])
       if(!quiet)
         fprintf(stderr, "%s: Query has a graph result:\n", program);
 
-      serializer=raptor_new_serializer(serializer_syntax_name);
+#ifdef RAPTOR_V2_AVAILABLE
+      serializer = raptor_new_serializer_v2(raptor_world_ptr, serializer_syntax_name);
+#else
+      serializer = raptor_new_serializer(serializer_syntax_name);
+#endif
       if(!serializer) {
         fprintf(stderr, "%s: Failed to create raptor serializer type %s\n",
                 program, serializer_syntax_name);
@@ -1030,16 +1183,25 @@ main(int argc, char *argv[])
 
  tidy_setup:
 
-  if(source_uris)
-    raptor_free_sequence(source_uris);
+  if(data_source_uris)
+    raptor_free_sequence(data_source_uris);
+  if(named_source_uris)
+    raptor_free_sequence(named_source_uris);
+#ifdef RAPTOR_V2_AVAILABLE
+  if(base_uri)
+    raptor_free_uri_v2(raptor_world_ptr, base_uri);
+  if(uri)
+    raptor_free_uri_v2(raptor_world_ptr, uri);
+#else
   if(base_uri)
     raptor_free_uri(base_uri);
   if(uri)
     raptor_free_uri(uri);
+#endif
   if(free_uri_string)
     raptor_free_memory(uri_string);
 
-  rasqal_finish();
+  rasqal_free_world(world);
   
   return (rc);
 }
